// LightMapSeq_dlg.cpp : implementation file
//

#include "stdafx.h"
#include "atLightMapSequence.h"
#include "LightMapSeq_dlg.h"

#include <Zeolite.h>

#include <zColour.h>
#include <zList.h>
#include <zFunc.h>
#include <zDirSel.h>

#include <zfile.h>

#include <helper/zapp.h>

// CLightMapSeq_dlg dialog

IMPLEMENT_DYNAMIC(CLightMapSeq_dlg, CDialog)
CLightMapSeq_dlg::CLightMapSeq_dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLightMapSeq_dlg::IDD, pParent)
	, m_FileName(_T(""))
	, m_OpenDirFlag(TRUE)
{
	m_FileName = _T("[Output folder name]");
}

CLightMapSeq_dlg::~CLightMapSeq_dlg()
{
}

void CLightMapSeq_dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_NEW_BUTTON, m_NewEditBtnCtrl);
	DDX_Control(pDX, IDC_DEL_BUTTON, m_DelBtnCtrl);
	DDX_Control(pDX, IDC_LIST1, m_ListCtrl);
	DDX_Text(pDX, IDC_FILENAME_EDIT, m_FileName);
	DDX_Check(pDX, IDC_OPENDIR_CHECK, m_OpenDirFlag);
}


BEGIN_MESSAGE_MAP(CLightMapSeq_dlg, CDialog)
	ON_BN_CLICKED(IDC_NEW_BUTTON, OnBnClickedNewButton)
	ON_BN_CLICKED(IDC_DEL_BUTTON, OnBnClickedDelButton)
	ON_LBN_SELCHANGE(IDC_LIST1, OnLbnSelchangeList1)
	ON_LBN_DBLCLK(IDC_LIST1, OnLbnDblclkList1)
	ON_BN_CLICKED(IDC_BROWSE_BUTTON, OnBnClickedBrowseButton)
END_MESSAGE_MAP()


// CLightMapSeq_dlg message handlers

void CLightMapSeq_dlg::OnBnClickedNewButton() {
	int index = m_ListCtrl.GetCurSel();

	if(index==LB_ERR)
		index = 0;

	index--;

	if(index>=0) {
		EditItem(index);
		return;
	}

	CzList run;

	run.Attach(m_List.CreateItem(VarID_varlist, NULL));

	// copy the preceding entry
	if(m_List.nItems()>1) { // 1 includes item created above
		run.CopyValue(m_List.GetItem(m_List.nItems()-2));
	} else {

		CzColour col;

		col.Attach(run.CreateItem(VarID_colour, "Sun"));
		col.SetRGB(255, 255, 255);

		col.Attach(run.CreateItem(VarID_colour, "Amb"));
		col.SetRGB(255, 255, 255);

		col.Attach(run.CreateItem(VarID_colour, "Spec"));
		col.SetRGB(255, 255, 255);

		float f;

		f = 0;
		run.SetItemValue("Azim", VarID_float, &f); 

		f = 30.0f;
		run.SetItemValue("Elev", VarID_float, &f);

		f = 1.0f;
		run.SetItemValue("Brightness", VarID_float, &f);

		f = 0.6f;
		run.SetItemValue("SunAmbRatio", VarID_float, &f);

		bool b;

		b = true;
		run.SetItemValue("UseWM", VarID_bool, &b);

		b = true;
		run.SetItemValue("ShadowFlag", VarID_bool, &b);

		b = false; // done in normal map
		run.SetItemValue("BumpMapping", VarID_bool, &b);

		b = true;
		run.SetItemValue("SpecularFlag", VarID_bool, &b);

		b = false;
		run.SetItemValue("MosaicFlag", VarID_bool, &b);

		int i;

		i = 512;
		run.SetItemValue("MosaicSize", VarID_int, &i);

		i = 1;
		run.SetItemValue("LM_in_HF", VarID_int, &i);

		vector3f v;
		v.x = 10.0f;
		v.y = 20.0f;
		v.z = 50.0f;
		run.SetItemValue("AbsRGB", VarID_vector3f, &v);

		b = true;
		run.SetItemValue("DoFresnel", VarID_bool, &b);

		f = 1.0f;
		run.SetItemValue("DefaultTx", VarID_float, &f);
	}

	if(!EditItem(m_List.nItems()-1)) {
		DelItem(m_List.nItems()-1);
	} else {
		PrepareButtons();
	}
}

void CLightMapSeq_dlg::OnBnClickedDelButton() {
	int index = m_ListCtrl.GetCurSel();

	index--;

	if(index>=0) DelItem(index);
}

void CLightMapSeq_dlg::OnLbnSelchangeList1() {
	PrepareButtons();
}

void CLightMapSeq_dlg::OnLbnDblclkList1() { OnBnClickedNewButton(); }

bool CLightMapSeq_dlg::EditItem(int index) {

	CzVar v(m_List.GetItem(index));
	if(v.EditUI("Edit run settings")) {
		PrepareList();
		return true;
	}

	return false;
}

void CLightMapSeq_dlg::DelItem(int index) { 
	m_List.DelItem(index); 
	PrepareList();
	PrepareButtons();
}

void CLightMapSeq_dlg::PrepareList() {
	m_ListCtrl.ResetContent();

	m_ListCtrl.AddString("<< New light map >>");

	CzList TempList;

	CString TempStr;

	float Azim, Elev;

	for(int i=0; i<m_List.nItems(); i++) {
		TempList.Attach(m_List.GetItem(i));

		zvar_GetValueEx(TempList.GetItem("Elev"), VarID_float, &Elev);
		zvar_GetValueEx(TempList.GetItem("Azim"), VarID_float, &Azim);

		TempStr.Format("Elev = %.0f, Azim = %.0f", Elev, Azim);
		m_ListCtrl.AddString(TempStr);
	}
}

void CLightMapSeq_dlg::PrepareButtons() {
	int index = m_ListCtrl.GetCurSel();

	if(index>0) {
		m_NewEditBtnCtrl.SetWindowText("Edit");
		m_DelBtnCtrl.EnableWindow(true);
	} else {
		m_NewEditBtnCtrl.SetWindowText("New");
		m_DelBtnCtrl.EnableWindow(false);
	}
}
BOOL CLightMapSeq_dlg::OnInitDialog() {
	CDialog::OnInitDialog();

	PrepareList();
	PrepareButtons();

	// get icon
	HICON hIcon = zapp_GetIcon();
	if(hIcon) {
		SetIcon(hIcon, true);
	}

	m_LinkCtrl.CreateLink(this, "On-line help", "http://www.bundysoft.com/wiki/doku.php?id=plugins:calc:atLightMapSequence");
	m_LinkCtrl.ShowLink();
	m_LinkCtrl.SetPosDLU(7, 6, SLC_LEFT|SLC_BOTTOM);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CLightMapSeq_dlg::OnBnClickedBrowseButton() {

	UpdateData(TRUE);

    LPCSTR lpFileName = NULL;
	if(m_FileName.CompareNoCase("[Output folder name]")!=0)
		lpFileName = (LPCSTR)m_FileName;

	CzDirSel DS;
	DS.Init(lpFileName);
	if(!DS.EditUI("Select output folder")) {
		return;
	}

	CzStr s;
	DS.GetCurSelA(s);
	m_FileName.SetString(s.GetText());

	UpdateData(FALSE);
}

void CLightMapSeq_dlg::OnOK() {
	
	UpdateData(TRUE);

	bool NameOK = true;
	if(0==m_FileName.CompareNoCase("[Output folder name]")) {
		NameOK = false;
	} else
	if(0==m_FileName.GetLength()) {
		NameOK = false;
	}

	if(!zfile_IsAbsolutePath(m_FileName)) {
		NameOK = false;

		CStringA UserDir;
		if(UserDir.GetEnvironmentVariableA("USERPROFILE")) {
			CStringA TempStr;
			TempStr.Format("%s\\%s", UserDir, m_FileName);
			m_FileName.SetString(TempStr);
		}
	}

	if(!NameOK) {
		OnBnClickedBrowseButton();
		return;
	}

	CDialog::OnOK();
}
