/*---------------------------------------------------------------------------------
|     Project | atLightMapSequence
| Description | A plugin for L3DT to generate a sequence of light maps from user-
|             | specified settings. 
|      Author | Aaron Torpy (aaron@bundysoft.com)
|   Copyright | (C) Copyright 2007-2011 Aaron Torpy. All rights reserved.
|         URL | http://www.bundysoft.com/wiki/doku.php?id=plugins:calc:atLightMapSequence
\---------------------------------------------------------------------------------*/

#include <stdafx.h>
#include "atLightMapSequence.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include <Zeolite.h>

#include <zList.h>
#include <zVar.h>
#include <zStr.h>
//#include <zMap.h>
#include <zFormat.h>
#include <zFunc.h>
#include <zProgBox.h>
#include <zComboSel.h>
#include <zColour.h>
//#include <zBuffer.h"

#include <zmenu.h>
#include <zproj.h>
#include <helper/zbackup.h>

#include <helper/zcalcman.h>

#include "LightMapSeq_dlg.h"

// CatLightMapSequenceApp

BEGIN_MESSAGE_MAP(CatLightMapSequenceApp, CWinApp)
END_MESSAGE_MAP()


// CatLightMapSequenceApp construction

CatLightMapSequenceApp::CatLightMapSequenceApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CatLightMapSequenceApp object

CatLightMapSequenceApp theApp;


// CatLightMapSequenceApp initialization

BOOL CatLightMapSequenceApp::InitInstance()
{
	CWinApp::InitInstance();

	return TRUE;
}

zeoexport bool __stdcall zeoInitPlugin() {

	CzFunc func;

	CzList args;
	args.CreateItem(VarID_varlist, "RunList");
	args.CreateItem(VarID_string, "OutputDir");
	args.CreateItem(VarID_uint, "Flags");
	func.Load("calc.LM.MultiLM", "ExtMultiLM", VarID_int, args);
	
	// add menu options
	func.Load("UI.LightMapSequence.MultiLM_UI", "ExtMultiLM_UI", VarID_void, NULL);
	func.Load("UI.LightMapSequence.ShowWikiPage", "ExtWikiPage", VarID_void, NULL);
	func.Load("UI.LightMapSequence.About", "ExtAbout", VarID_void, NULL);
	zmenu_InsertItem("UI.LightMapSequence.MultiLM_UI", "Light map sequence");
	zmenu_InsertItem("UI.LightMapSequence.ShowWikiPage", "Visit project page");
	zmenu_InsertItem("UI.LightMapSequence.About", "About");

	return true;
}

bool __stdcall ExtAbout(ZVAR hReturnVar, ZLIST hArgList) {
	AfxMessageBox("LightMapSequence is a plugin that generates a sequence of light maps automatically.\r\n\r\nCopyright (C) 2007-2010 Aaron Torpy. All rights resserved.", MB_ICONINFORMATION);
	return true;
}
bool __stdcall ExtWikiPage(ZVAR hReturnVar, ZLIST hArgList) {
	ShellExecute(NULL, "Open", "http://www.bundysoft.com/wiki/doku.php?id=plugins:calc:atLightMapSequence", NULL, NULL, SW_NORMAL);
	return true;
}

bool __stdcall ExtMultiLM_UI(ZVAR hReturnVar, ZLIST hArgList) {

	// check that the HF and TN are OK
	ZMAP pHF = zproj_GetMap("HF");
	if(!pHF) {
		zeoReportError("atLightMapSequence::ExtMultiLM_UI error:\r\n - cannot get heightfield");
		return false;
	}
	if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
		AfxMessageBox("The heightfield is not initialised.", MB_ICONERROR);
		return true; // exit quietly
	}
	ZMAP pTN = zproj_GetMap("TN");
	if(!pTN) {
		zeoReportError("atLightMapSequence::ExtMultiLM_UI error:\r\n - cannot get normal map");
		return false;
	}
	if(MAP_TerrainNormals!=zmap_GetMapType(pTN)) {
		AfxMessageBox("The normal map is not initialised.", MB_ICONERROR);
		return true; // exit quietly
	}

	CzFunc fMultiLM;
	if(!fMultiLM.GetFunc("calc.LM.MultiLM")) {
		zeoReportError("atLightMapSequence::ExtMultiLM_UI error:\r\n - cannot get function");
		return false;
	}

	if(zCalcMan_IsBusy()) {
		zeoReportError("atLightMapSequence::ExtMultiLM_UI error:\r\n - project is busy");
		return false;
	}

	CLightMapSeq_dlg dlg;
	if(dlg.DoModal()!=IDOK) {
		return true;
	}


	/* old way
	
	zvar_CopyValue(fMultiLM.GetArgA("RunList"), dlg.m_List);

	zstr_SetText(fMultiLM.GetArgA("OutputDir"), dlg.m_FileName);

	unsigned long Flags = 0;
	if(dlg.m_OpenDirFlag)	Flags  = Flags|0x00000001;
	fMultiLM.SetArgValueA("Flags", VarID_uint, &Flags);

	fMultiLM.Execute2();
	*/

	// threaded way
	CzList args;
	if(args.Attach(zvar_GetSharedVar("LightMapSequence.args"))) {
		args.DelAll();
	} else
	if(!args.CreateShared("LightMapSequence.args")) {
		zeoReportError("atLightMapSequence::cannot create args");
		return false;
	}

	zvar_CopyValue(args.CreateItem(VarID_varlist, "RunList"), dlg.m_List);

	zstr_SetText(args.CreateItem(VarID_string, "OutputDir"), dlg.m_FileName);

	unsigned long Flags = 0;
	if(dlg.m_OpenDirFlag)	Flags  = Flags|0x00000001;
	args.SetItemValueA("Flags", VarID_uint, &Flags);

	//args.EditUI();

	args.Release();

	if(!zCalcMan_RunCalcScript("calc.LM.MultiLM LightMapSequence.args.RunList LightMapSequence.args.OutputDir LightMapSequence.args.Flags")) {
		zeoReportError("atLightMapSequence::ExtMultiLM_UI error:\r\n - cannot run calculation");
		return false;
	}


	return true;
}

bool __stdcall ExtMultiLM(ZVAR hReturnVar, ZLIST hArgList) {

	int rval = -1;
	if(!zvar_SetValueEx(hReturnVar, VarID_int, &rval)) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot set rval");
		return false;
	}

	CzList args;
	args.Attach(hArgList);

	CzFunc fGenLM;
	if(!fGenLM.GetFunc("calc.LM.GenMap")) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot get function");
		return false;
	}

	CzList ProjSettings;
	ProjSettings.Attach(zproj_GetSettingsList());

	CzList ProjSettingsLM;
	ProjSettingsLM.Attach(ProjSettings.GetOrCreateItem(VarID_varlist, "LM"));

	//CzProgBox prog;
	//prog.SetTitle("Generating light maps");
	//prog.ShowWnd();

	CString TempStr;

	CzList item;
	bool IsMosaic;

	ZMAP hMap;

	CzFormat format;
	if(!format.GetActiveFormat("LM", MAP_RGB)) {
	//if(!format.GetByExt("LM", MAP_24bit, "jpg")) {
		return false;
	}

	CzList RunList;
	if(!RunList.Attach(args.GetItem("RunList"))) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot get run list");
		return false;
	}

	if(!RunList.nItems()) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - invalid item count");
		return false;
	}

	CzStr DirName;
	if(!DirName.Attach(args.GetItem("OutputDir"))) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot get output dir");
		return false;
	}

	unsigned long Flags = 0;
	if(!args.GetItemValue("Flags", VarID_uint, &Flags)) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot get flags");
		return false;
	}

	// store backup
	if(zbackup_GetAutoBackupFlag()) {
		zbackup_BackupMaps("LM|SpecLM", "Light map sequence", 0, "view.ShowMap \"LM\"");
	}

	bool OpenDirFlag = false;
	if(Flags & 0x00000001) {
		OpenDirFlag = true;
	}

	CStringA FileName;

	rval = 0;
	
	for(int i=0; i<RunList.nItems(); i++) {
		if(!item.Attach(RunList.GetItem(i))) {
			zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot get run item");
			return false;
		} 

		item.GetItemValue("MosaicFlag", VarID_bool, &IsMosaic);

		//prog.SetProgress((__int64)i, (__int64)args.nItems());

		if(!ProjSettingsLM.CopyValue((ZVAR)item)) {
			zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot copy project settings");
			return false;
		}

		if(!fGenLM.Execute2()) {
			zeoReportError("atLightMapSequence::MultiLM error:\r\n - call to GenLM failed");
			return false;
		}
		bool TempBool = false;
		if(!fGenLM.m_rval.GetValue(VarID_bool, &TempBool)) {
			zeoReportError("atLightMapSequence::MultiLM error:\r\n - call to GenLM returned false");
			return false;
		}
		
		rval++;

		// now go on and export light map
		hMap = zproj_GetMap("LM");
		if(!IsMosaic) {
			FileName.Format("%s\\LM-%d.%s", DirName.GetText(), i+1, format.GetExt());
			if(!zmap_SaveFile(hMap, FileName, format.GetZVAR(), false, true)) {
				CStringA TempStr;
				TempStr.Format("atLightMapSequence::MultiLM error:\r\n - cannot save map to '%s'", FileName);
				zeoReportError(TempStr);
				return false;
			}
		} else {
			FileName.Format("%s\\LM-%d\\LM-%d.mmf", DirName.GetText(), i+1, i+1);
			CzStr MapName;
			zmap_GetNameInProject(hMap, MapName.GetZVAR());
			if(!zmap_ExportMosaic(hMap, FileName, format.GetZVAR(), zmap_GetWidth(hMap), zmap_GetHeight(hMap), zmap_GetTileSize(hMap), MapName.GetText())) {
				CStringA TempStr;
				TempStr.Format("atLightMapSequence::MultiLM error:\r\n - cannot save map to '%s'", FileName);
				zeoReportError(TempStr);
				return false;
			}
		}

		if(i==0) {
			zeoReportError("atLightMapSequence report:");
		}
		CStringA TempStr;
		TempStr.Format("Light map #%d saved to '%s'", i+1, FileName);
		zeoReportError(TempStr);

		bool SpecFlag = true;
		if(item.SetItemValue("SpecularFlag", VarID_bool, &SpecFlag)) {
			if(SpecFlag) {
				// now go on and export light map
				hMap = zproj_GetMap("SpecLM");
				if(!IsMosaic) {
					FileName.Format("%s\\SpecLM-%d.%s", DirName.GetText(), i+1, format.GetExt());
					if(!zmap_SaveFile(hMap, FileName, format.GetZVAR(), false, true)) {
						CStringA TempStr;
						TempStr.Format("atLightMapSequence::MultiLM error:\r\n - cannot save map to '%s'", FileName);
						zeoReportError(TempStr);
						return false;
					}
				} else {
					FileName.Format("%s\\SpecLM-%d\\SpecLM-%d.mmf", DirName.GetText(), i+1, i+1);
					CzStr MapName;
					zmap_GetNameInProject(hMap, MapName.GetZVAR());
					if(!zmap_ExportMosaic(hMap, FileName, format.GetZVAR(), zmap_GetWidth(hMap), zmap_GetHeight(hMap), zmap_GetTileSize(hMap), MapName.GetText())) {
						CStringA TempStr;
						TempStr.Format("atLightMapSequence::MultiLM error:\r\n - cannot save map to '%s'", FileName);
						zeoReportError(TempStr);
						return false;
					}
				}

				CStringA TempStr;
				TempStr.Format("Specular light map #%d saved to '%s'", i+1, FileName);
				zeoReportError(TempStr);
			}
		}
	}

	//prog.HideWnd();

	TempStr.Format("atLightMapSequence generated %d maps OK", rval);
	zeoReportError(TempStr);

	if(OpenDirFlag) {
		// let's have a look at the output
		ShellExecute(NULL, "Open", (char*)DirName, NULL, NULL, SW_NORMAL);
	}

	if(!zvar_SetValueEx(hReturnVar, VarID_int, &rval)) {
		zeoReportError("atLightMapSequence::MultiLM error:\r\n - cannot set rval (2)");
		return false;
	}

	return true;
}
